<?php
namespace TriNetAssess;

use GuzzleHttp\ClientInterface;
use Psr\Http\Message\ResponseInterface;

// This class is named as though it should be a generic OAuth 2
// flow handler (not specific to a host/resource). However, given
// the inability to configure the OAuth host and define the resource
// to be requested, it seems as this class is actually only responsable
// for handling the OAuth flow for requesting a single predefined
// resouce.
define('AUTHENTICATION_URI', 'http://example.org/oauth/token');
define('RESOURCE_URI',       'http://example.org/api/some_resource');

class OAuth2FlowAssess
{
    protected $clientIntf;
    
    function __construct(ClientInterface $clientIntf = null) {
        $this->setClient($clientIntf);
    }
    
    // Getter and setter for the HTTP client
    function getClient() {
        if (!$this->clientIntf) {
            throw new AssessmentException('Client is unset. Use setClient().');
        }
        
        return $this->clientIntf;
    }
    function setClient(ClientInterface $clientIntf = null) {
        $this->clientIntf = $clientIntf;
        return $this->clientIntf;
    }
    
    
    // attempt to authenticate and retrieve an authorized access token
    function sendAuthenticationRequest($clientId, $clientSecret) {
        $clientIntf = $this->getClient();
        
        // make the HTTP request
        $response = $clientIntf->post(AUTHENTICATION_URI, [
            'form_params' => [
                'grant_type'    => 'client_credentials',
                'client_id'     => $clientId,
                'client_secret' => $clientSecret
            ]
        ]);
        
        // NOTE: This interface is very strange. It seems as though
        // the responsibility of parsing and validating the HTTP response
        // from this request is part of sendAuthorizedRequest.
        return $response;
        
        /*
        // handle and parse the response
        $result;
        try {
            // ensure we got a 200 back
            if ($response->getStatusCode() !== 200) {
                throw new Exception('Non 200 response received.');
            }
            
            $result = OAuth2FlowAssess::parseAuthResponse($response);
        }
        catch (Exception $exp) {
            // TODO: more robust and detailed error handling
            throw new AssessmentException("Invalid Authentication response received. Status: $response->getStatusCode() Body:\n$response->getBody()", 0, $exp);
        }
        
        $accessToken = $result['accessToken'];
        $tokenType   = $result['tokenType'];
        
        return [
            'accessToken'  => $accessToken,
            'tokenType'    => $tokenType,
            'authResponse' => $response
        ];
        */
    }
    
    // get the resource
    // NOTE: The interface for this method does not make sense to me.
    // Why pass in an HTTP response object rather than the access token?
    // Why is this method responsible for parsing HTTP responses rather
    // than being agnostic of the source and dealing only with the access
    // token directly?
    // Also, the name of this method seems to imply that it can be used
    // to make any request. However, because the interface does not allow
    // any URI to be passed, this method actually requests a predefined
    // resource.
    function sendAuthorizedRequest(ResponseInterface $authResponse) {
        $clientIntf = $this->getClient();
        
        // parse the response
        $result;
        try {
            $result = OAuth2FlowAssess::parseAuthResponse($authResponse);
        }
        catch (Exception $exp) {
            throw new AssessmentException("Invalid Authentication response.", 0, $exp);
        }
        
        $accessToken = $result['accessToken'];
        $tokenType   = $result['tokenType'];
        
        // make the HTTP request
        $response = $clientIntf->post(RESOURCE_URI, [
            'headers' => [
                'Authorization' => "${tokenType} ${accessToken}"
            ]
        ]);
        
        // return the response
        return $response;
    }
    
    protected static function parseAuthResponse(ResponseInterface $response) {
        // parse the body as JSON
        try {
            $body = json_decode($response->getBody(), true);
        }
        catch (Exception $exp) {
            throw new \Exception('Response body is not valid JSON.', 0, exp);
        }
        
        // pull out access token data
        $accessToken = $body['access_token'];
        $tokenType   = $body['token_type'];
        
        if (empty($accessToken)) {
            throw new \Exception('Access token missing from body.');
        }
        if (empty($tokenType)) {
            throw new \Exception('Token type missing from body.');
        }
        
        return [
            'accessToken' => $accessToken,
            'tokenType'   => $tokenType
        ];
    }
}